Const Nodes%        = 200       ; number of stars to visualize (out of 3600)
Const Factor%		= 10

Include "inc/spanning.bb"
Include "inc/3600stars.bb"
Include "inc/milkyway.bb"

AppTitle "The "+Nodes+" Nearby Stars"

Init()
TConnection_InitAll()
TConnection_CreateSpanningTree(First TNode)

Graphics3D 800,600,32,2

Global wiref_piv = CreatePivot()
Global solid_piv = CreatePivot()

Const TurnSpeed#    = 4.00      ; cam turn speed
Const RollSpeed#    = 0.50      ; cam roll speed
Const CameraSpeed#  = 0.0001    ; cam move speed
Const RadarSize%    = 384       ; radar size in pixels
Const Maxrange#     = 10.0      ; Scanner Range in Lightyears
Const RadioScale#   = 90*Factor	; range of human radio signals

Const PoleWidth#    = 1.0       ; width of poles
Const BlipSize#     = 4.0       ; size of radar blip
Const VertScale#    = 1.0       ; height multiplicator of pole/blip
Const RadarBorder#  = 0.9       ; range within 1.0 to display

Const R1%=  0,G1%=128,B1%=192   ; grid lines color
Const R2%=  0,G2%= 16,B2%= 32   ; scanner background color
Const R3%=  0,G3%=128,B3%=255   ; scanner border glow color
Const R4%=  0,G4%=128,B4%=255   ; star connection colors

Const ALPHA#=0.125

Global WithinRange%

Type blip
    
    Field entity%,sun1%,sun2%,sun3%
    Field blip%
    Field pole%
	Field name$
    Field order$
	Field Constellation$
	Field x#,y#,z#
	Field distance#
	Field class$[2],dwarf$,temp%
    Field r%,g%,b%
    Field absmag#
	Field scale#
	Field inrange%
    
End Type

Global WIDTH%=GraphicsWidth()
Global HEIGHT%=GraphicsHeight()
Global SUNTEX%=CreateSunTexture(128)
Local TIMER%=CreateTimer(60)

; camera
Local CAM%=CreateCamera()
CameraRange CAM,0.01,20000*Factor

; setup radar
Local RADAR=InitRadar(RadarSize,512,16,CAM)

; setup objects
InitObjects(RADAR,Nodes)

; setup spanning tree connections
TConnections_Show()

; setup milkyway background
Global milkyway%=InitMilkyway(500,0.2,10,40,500,0,0,0,0,255,192,128,0.5,0,0,0,0)
EntityParent milkyway,solid_piv

; setup galactic plane
plane=CreateQuad(solid_piv,RadioScale,False,3,1+16,0,0,255,0.025)
RotateEntity plane,90,0,0

; setup radio range
mesh=CreateMesh()
surf=CreateSurface(mesh)
EntityFX mesh,1+2+16+32
line1=Line3D(mesh,-RadioScale,0,0,RadioScale,0,0,255,255,255,0.1)
line2=Line3D(mesh,0,-RadioScale,0,0,RadioScale,0,255,255,255,0.1)
line3=Line3D(mesh,0,0,-RadioScale,0,0,RadioScale,255,255,255,0.1)
sphere=CreateSphere(32,wiref_piv)
ScaleEntity sphere,RadioScale,RadioScale,RadioScale
EntityFX sphere,1+2+16+32
surf=GetSurface(sphere,1)
For v=0 To CountVertices(surf)-1
	VertexColor surf,v,255,0,255,0.05
Next

MoveMouse WIDTH/2,HEIGHT/2
PointEntity CAM,solid_piv

AppTitle "The "+Nodes+" Nearby Stars ("+WithinRange+" could listen to our radio signals)"


While Not KeyHit(1)
    
    Local range%=Maxrange,visible%
    
    ; move player
    Movement(CAM,1.25)
    
	PositionEntity milkyway,EntityX(CAM,1),EntityY(CAM,1),EntityZ(CAM,1)
	
	If KeyHit(28) Then names=1-names
	If KeyHit(57) Then wf=1-wf
	
	UpdateWorld
	
    ; Render wireframe objects.
	If wf=0 Then
		WireFrame 1
		ShowEntity wiref_piv
		HideEntity solid_piv
		HideEntity RADAR
		ShowEntity plane
		CameraClsMode CAM,1,1 : RenderWorld
	Else
		WireFrame 0
		ShowEntity wiref_piv
		HideEntity solid_piv
		HideEntity plane
		CameraClsMode CAM,1,1 : RenderWorld
	EndIf
	
    ; Render solid objects.
    WireFrame 0
	HideEntity wiref_piv
	ShowEntity solid_piv
	ShowEntity RADAR
    CameraClsMode CAM,0,0 : RenderWorld
	
	; update radar
    visible%=UpdateRadar(CAM,RADAR,RadarSize,range*Factor,90)
    
	CameraPick CAM,MouseX(),MouseY()
	Local b.blip,col%,d#
	Local output$="",out$
	Local spectrum$=""
	Local temp$=""
	Local distancesol$=""
    Local distanceship$=""
	Local absmag$=""
	Local inrange$="too far away"
    Local picked%=PickedEntity()
    Local f#
	
	For b.blip = Each blip
        
		; scale stars
		d=EntityDistance(b\entity,CAM)
		PointEntity b\entity,CAM
		f=Sqr(d*Sqr(b\scale))/25.0
		ScaleEntity b\entity,f,f,f
		EntityRadius b\entity,f*2,f*2
		
		; star picker, fader
		If EntityInView(b\entity,CAM) Then
			
			If names=0 Then
			
				col=Normalize(d,range*0.9*Factor,range*Factor,255,0)
				If col<0 Then col=0 Else If col>255 Then col=255
				Color 0,col,0
                
				If d<range*Factor Then
					out=b\name
					If d<range*0.5*Factor Then out=b\name+" "+b\order
					CameraProject CAM,b\x,b\y,b\z
					px=ProjectedX()
					py=ProjectedY()
					Text px,py-14,out,1,1
                    
				EndIf
				
			EndIf
				
			If picked=b\entity Then
					
				output=b\name
				spectrum$=b\class[0]+","+b\class[1]+","+b\class[2]
				temp=b\temp+"C"
				distancesol=b\distance+" LY"
				distanceship=d/Factor+" LY"
				absmag=b\absmag
				
				If b\inrange=1 Then inrange="yes"
                
				If b\dwarf<>"" Then spectrum=spectrum+" ["+b\dwarf+" dwarf]"
				
				CameraProject CAM,b\x,b\y,b\z
				px=ProjectedX()
				py=ProjectedY()
				
				ps=9
				
				Color 0,128,0
				Oval px-ps,py-ps,2*ps,2*ps,0
				Line px-(2*ps),py,px-ps,py
				Line px+(2*ps),py,px+ps,py
				Line px,py-(2*ps),px,py-ps
				Line px,py+(2*ps),px,py+ps
				
				
			EndIf
				
		EndIf
			
	Next
	
	WaitTimer TIMER
    
	If picked Then
		
		Color 255,255,255
		Text 0, 0,"Star Name..........: "+output
		Text 0,15,"Distance from Sun..: "+distancesol
        Text 0,30,"Distance from here.: "+distanceship
		Text 0,45,"Spectral class.....: "+spectrum
		Text 0,60,"Surface Temperature: "+temp
        Text 0,75,"Absolute Magnitude.: "+absmag
		Text 0,90,"Within Radio Range.: "+inrange
		
    EndIf
	
    Flip 0
    
Wend

End

Function Init()
	
	Local d%,x#,y#,z#
	
	Restore Stars
	
	For i=1 To Nodes
		
		Read d,d,d,d,d,d,d,x,y,z
		Read d,d,d,d,d,d,d
		
		Local n.TNode = New TNode
		
		n\id = i
		n\pos[0] = x*Factor
		n\pos[1] = z*Factor
		n\pos[2] = y*Factor
		NODE_ARRAY(i-1) = n
		
	Next
	
End Function

Function TConnections_Show()
	For i% = 0 To Nodes-1
		c.TConnection = CurrentConnectionsArray(i)
		If c<>Null
			If c\entity FreeEntity c\entity
				c\entity = Line3D(mesh,c\node[0]\pos[0],c\node[0]\pos[1],c\node[0]\pos[2],c\node[1]\pos[0],c\node[1]\pos[1],c\node[1]\pos[2],R4,G4,B4,ALPHA)
			EndIf
		Next
End Function

; 3D Linie zeichnen
Function Line3D(mesh,x0#,y0#,z0#,x1#,y1#,z1#,r%=255,g%=255,b%=255,a#=1.0)
	
	If mesh = 0
		mesh = CreateMesh()
		surf = CreateSurface(mesh)
		EntityFX mesh,1+2+16+32
	Else
		last_surf = CountSurfaces(mesh)
		surf = GetSurface(mesh,last_surf)
		If CountVertices(surf) > 30000 Then surf = CreateSurface(mesh)
	End If
	
    v1 = AddVertex(surf,x1,y1,z1)
	v2 = AddVertex(surf,x0,y0,z0)
	AddTriangle surf,v1,v1,v2
	
	VertexColor surf,v1,r,g,b,a
	VertexColor surf,v2,r,g,b,a
	
	Return mesh
End Function

Function Distance3D#(x1#,y1#,z1#,x2#,y2#,z2#)
	
	Local x#=x1-x2
	Local y#=y1-y2
	Local z#=z1-z2
	
	Return Sqr((x*x)+(y*y)+(z*z))
	
End Function


; populate scene with dummy objects
Function InitObjects(radar%,number%=100)
    
    Local i%,b.blip
	Local name$,ab1$,ab2$,gliese%,num%,magnitude#
	Local add1$,add2$
	
	Local entity%=CreateQuad(solid_piv,1,SUNTEX,3,1+8)
	
	Restore Stars
	
	For i=1 To number
		
		b.blip = New blip
		
		Read name,ab1,ab2,gliese,num,b\order,b\Constellation,b\x,b\z,b\y
		Read magnitude,b\absmag,b\class[0],b\class[1],b\class[2],b\dwarf,b\temp
		
		; name creation
		If name<>"" Then
			
			; use standard name
			b\name=name
            
            If Len(name)<3 Then b\name=name+" "+Constellation(b\Constellation)
			
		Else
			
			add1=""
			add2=""
            
            If ab1<>"" Then
				
				; build alphabeta-constellation name	
				If ab2>0 Then add1=" "+ab2
				add2=" "+Constellation(b\Constellation)
				b\name=AlphaBeta(ab1)+add1+add2
				
			Else
				
				If gliese>0 Then
					
					; build gliese name
					b\name="Gliese "+gliese
					
				Else
					
					; build number-constellation name
					If num>0 Then b\name=num+" "+Constellation(b\Constellation)
					
				EndIf
				
			EndIf
			
		EndIf
        
        b\x=b\x*Factor
		b\y=b\y*Factor
		b\z=b\z*Factor
        
		b\distance=Distance3D(0,0,0,b\x/Factor,b\y/Factor,b\z/Factor)
		b\scale=3.0/Exp((b\absmag-2))*Factor
		
		b\inrange=0
		
		If b\distance*Factor<RadioScale Then
			WithinRange=WithinRange+1
			b\inrange=1
		EndIf
		
		b\r=255
		b\g=255
		b\b=255
		
		Select b\class[0]
				
			Case "O":	b\r=155 : b\g=176 : b\b=255
			Case "B":	b\r=170 : b\g=191 : b\b=255
			Case "A":	b\r=202 : b\g=215 : b\b=222
			Case "F":	b\r=248 : b\g=247 : b\b=255
			Case "G":	b\r=255 : b\g=244 : b\b=234
			Case "K":	b\r=255 : b\g=210 : b\b=161
			Case "M":	b\r=255 : b\g=204 : b\b=223
		
		End Select
        
		b\entity=CopyEntity(entity,solid_piv)
		b\blip=CreateQuad(radar)
		b\pole=CreateQuad(radar)
		
		EntityColor b\entity,b\r,b\g,b\b
		
        ; make pickable for details
        EntityPickMode b\entity,1
		
		; create the entity itself
        PositionEntity b\entity,b\x,b\y,b\z
		ScaleEntity b\entity,0.00001,0.00001,0.00001
		
        ; creates a radar blip
        EntityFX b\blip,1
        EntityOrder b\blip,-100
		RotateMesh b\blip,90,0,0
		EntityTexture b\blip,SUNTEX
		EntityBlend b\blip,3
        
        ; creates a blip pole
        EntityFX b\pole,1
		EntityColor b\pole,128,128,128
        EntityOrder b\pole,-50
		RotateMesh b\pole,90,0,0
		
		HideEntity entity
		
    Next
    
End Function

; simple spaceship movement
Function Movement(cam%,sensitivity#=1.0)
    
    Local roll#,cx#,cz#,tx#,ty#,multi1%=1,multi2%=1
    
    ; arrows = move / SHIFT or LMB = Turbo
    cz=(KeyDown(200)-KeyDown(208))*CameraSpeed
	roll=(KeyDown(203)-KeyDown(205))*RollSpeed
    If MouseDown(1) Then multi1=100
	If MouseDown(2) Then multi2=5000
    
    tx=Normalize(MouseX(),0,WIDTH , 1,-1)
    ty=Normalize(MouseY(),0,HEIGHT,-1, 1)
    
    If ty<0 Then ty=(Abs(ty)^sensitivity)*-1 Else ty=ty^sensitivity
    If tx<0 Then tx=(Abs(tx)^sensitivity)*-1 Else tx=tx^sensitivity
    
    TurnEntity cam,ty*TurnSpeed,tx*TurnSpeed,roll*TurnSpeed
    MoveEntity cam,cx*(multi1*multi2),0,cz*(multi1*multi2)
	
End Function

; creates a nice elite stype radar sprite at screen bottom
Function InitRadar(scale#,size%=512,quads%=8,parent%=False,sharp#=1.0,blur#=50.0)
    
    Local RADAR%=CreateQuad(solid_piv)
	EntityParent RADAR,parent
	
    Local tex1%=CreateGridTexture(size,quads)
    Local tex2%=CreateGlowTexture(size,sharp,blur)
    
    ; apply textures to radar quad
    EntityFX RADAR,1+16
    ScaleEntity RADAR,scale,scale,scale
    PositionEntity RADAR,0,-384,WIDTH
    TextureBlend tex1,2
    TextureBlend tex2,3
    EntityTexture RADAR,tex1,0,1
    EntityTexture RADAR,tex2,0,2
    EntityBlend RADAR,1
    
    Return RADAR
    
End Function

; updates the radar scene
Function UpdateRadar(cam%,radar%,scale#=128.0,range#=256.0,angle#=60.0)
    
    Local b.blip
    Local d#
    Local rx#,ry#,rz#
    
    Local halfscale#=scale/2.0
    Local doublescale#=scale*2.0
    Local ps#=PoleWidth/scale
    Local bs#=BlipSize/halfscale
    Local div#=((scale/2.0)/range)*RadarBorder
    
    Local visible%=0
    
    For b.blip = Each blip
        
		; get distance to object
        d=EntityDistance(b\entity,cam)
		
		HideEntity b\pole
        HideEntity b\blip
		
		If d<range*2 Then
			
            ; within scanner range?
			If d<range Then
            
				a#=Normalize(d,range*0.9,range,1,0)
				EntityAlpha b\blip,a
				EntityAlpha b\pole,a
				
				visible=visible+1
			
				; show blip and its pole
				ShowEntity b\pole
				ShowEntity b\blip
            
				; transform position
				TFormPoint (b\x,b\y,b\z,0,cam)
				rx=TFormedX()/halfscale*div
				rz=TFormedZ()/halfscale*div
				ry=TFormedY()*div
            
				; reposition/scale blip
				PositionEntity b\blip,0,0,0
				TranslateEntity b\blip,rx,rz,ry/scale*VertScale
				ScaleEntity b\blip,bs,bs,bs
            
				; reposition/scale pole
				PositionEntity b\pole,0,0,0
				ScaleEntity b\pole,ps,ps,Abs(ry/doublescale*VertScale)
				TranslateEntity b\pole,rx,rz,ry/doublescale*VertScale
				
				EntityColor b\blip,b\r,b\g,b\b
            
            EndIf
			
        EndIf
        
    Next
    
    ; radar always points to cam
    RotateEntity radar,angle,0,0
    
    Return visible
    
End Function

; advanced quad creation
Function CreateQuad(parent%=False,scale#=1.0,tex%=False,blend%=False,fx%=False,r%=255,g%=255,b%=255,a#=1.0)
	
	Local mesh%=CreateMesh()
	Local surf%=CreateSurface(mesh)
	
	Local v0%=AddVertex(surf, 1, 1,0,0,0)
	Local v1%=AddVertex(surf,-1, 1,0,1,0)
	Local v2%=AddVertex(surf,-1,-1,0,1,1)
	Local v3%=AddVertex(surf, 1,-1,0,0,1)
	
	AddTriangle surf,v0,v1,v2
	AddTriangle surf,v0,v2,v3
	
	If parent Then EntityParent mesh,parent
	If fx Then EntityFX mesh,fx
	If tex Then EntityTexture mesh,tex
	If blend Then EntityBlend mesh,blend
	
	EntityColor mesh,r,g,b
	EntityAlpha mesh,a
	
	VertexColor surf,v0,r,g,b,a
	VertexColor surf,v1,r,g,b,a
	VertexColor surf,v2,r,g,b,a
	VertexColor surf,v3,r,g,b,a
	
	ScaleEntity mesh,scale,scale,scale
	
	Return mesh
	
End Function

; create procedural glowing border texture
Function CreateGlowTexture(size%=512,sharpness#=1.0,blur#=50.0)
    
    Local strength#=512*1.0/size*sharpness
    Local tex%=CreateTexture(size,size,3)
    Local tb%=TextureBuffer(tex)
    
    Local x%,y%,i#,j%,col%,rgb%,rc%,gc%,bc%
    
    LockBuffer tb
    
    For x=0 To size-1
        
        For y=0 To size-1
            
            ; clear colors
            rgb=0*$1000000+0*$10000+0*$100+0
            WritePixelFast x,y,rgb,tb
            
        Next
        
    Next
    
    For j=1 To (size/2)-2
        
        ; exponential falloff
        col=j*strength/Exp((((size/2)-2)-j)*(strength/blur))
        
        If col>224 Then col=Normalize(col,224,255,224,0)
        If col<0 Then col=0
        
        ; multiply RGB with brightness
        rc=R3*col Shr 8
        gc=G3*col Shr 8
        bc=B3*col Shr 8
        
        rgb=255*$1000000+rc*$10000+gc*$100+bc
        
        For i=0 To 360 Step 0.05
            
            WritePixelFast ((size-2)/2.0)+(Sin(i)*j),((size-2)/2.0)+(Cos(i)*j),rgb,tb
            
        Next
        
    Next
    
    UnlockBuffer tb
    
    Return tex
    
End Function

; normalize a value
Function Normalize#(value#=128.0,vmin#=0.0,vmax#=255.0,nmin#=0.0,nmax#=1.0)
    
    Return ((value#-vmin#)/(vmax#-vmin#))*(nmax#-nmin#)+nmin#
    
End Function

; create radar grid texture
Function CreateGridTexture(size%=512,quads%=16)
    
    Local x%,y%,j%,i#,rgb%,col%,r%,g%,b%
    Local steps%=size/quads
    
    Local tex%=CreateTexture(size,size,3+8)
    Local buffer%=TextureBuffer(tex)
    
    SetBuffer buffer
    
    ; background color
    Color R2,G2,B2
    Rect 0,0,size,size,1
    
    ; create grid
    Color R1,G1,B1
    For x=0 To quads-1
        For y=0 To quads-1
            Rect x*steps,y*steps,steps+1,steps+1,0
        Next
    Next
    
    Color 255,255,255
    
    LockBuffer buffer
    
    ; create glowing border and delete pixels outside the circle
    For j=(size/2)-6 To size
        
        For i=0 To 360 Step 0.05
            
            x=((size-2)/2.0)+(Sin(i)*j)
            y=((size-2)/2.0)+(Cos(i)*j)
            
            col=0
            If j<=(size/2) Then col=Normalize(j,(size/2)-6,(size/2),255,0)
            
            ; multiply RGB with brightness
            r=R3*col Shr 8
            g=G3*col Shr 8
            b=B3*col Shr 8
            
            rgb=0*$1000000+r*$10000+g*$100+b
            
            If x>=0 And x<size And y>=0 And y<size Then
				
                WritePixelFast x,y,rgb,buffer
                
            EndIf
            
        Next
        
    Next
	
    UnlockBuffer buffer
    
    SetBuffer BackBuffer()
    
    Return tex
    
End Function

Function CreateSunTexture(size%=512)
	
	Local tex%=CreateTexture(size,size,3)
	Local tb%=TextureBuffer(tex)
	
	Local i#,j%,col%,rgb%
	Local x%,y%,xx%,yy%
	Local r%,g%,b%,a%
	
	LockBuffer tb
	
	For j=0 To (size/2)-1
		
		col=255-Normalize(j,0,(size/2)-1,0,255)
		If col>255 Then col=255
		rgb=col*$1000000+col*$10000+col*$100+col
		
		For i=0 To 360 Step 0.05
			
			WritePixelFast (size/2)+(Sin(i)*j),(size/2)+(Cos(i)*j),rgb,tb
			
		Next
		
	Next
	
	UnlockBuffer tb
	
	Local tempcam%=CreateCamera()
	Local tempsun%=CreatePivot()
	
	Local sun0%=CreateQuad(tempsun,size/4.0,tex,3,1+8+16,255,255,255,1.00)
	Local sun1%=CreateQuad(tempsun,size/1.5,tex,3,1+8+16,255,192,128,1.00)
	Local sun2%=CreateQuad(tempsun,size/1.2,tex,3,1+8+16,255,255,224,0.75)
	Local sun3%=CreateQuad(tempsun,size/1.0,tex,3,1+8+16,255,255,224,0.50)
	
	PositionEntity tempsun,0,0,WIDTH
    
    RenderWorld
	
	LockBuffer BackBuffer()
	LockBuffer tb
	
	For x=0 To size-1
		
		For y=0 To size-1
			
			xx=(WIDTH/2)-(size/2)+x
			yy=(HEIGHT/2)-(size/2)+y
			
			rgb=ReadPixelFast(xx,yy,BackBuffer())
			
			r=(rgb And $ff0000)/$10000
			g=(rgb And $ff00)/$100
			b=(rgb And $ff)
			a=255
			
			If (r+g+b)/3 < 32 Then a=Normalize((r+g+b)/3,0,32,0,255)
			
			rgb=a*$1000000+r*$10000+g*$100+b
			
			WritePixelFast x,y,rgb,tb
			
		Next
		
	Next
	
	UnlockBuffer tb
	UnlockBuffer BackBuffer()
	
	RenderWorld
	Cls
	
	FreeEntity sun0
	FreeEntity sun1
	FreeEntity sun2
	FreeEntity sun3
	FreeEntity tempcam
	FreeEntity tempsun
	
	Return tex
	
End Function

; return long greek AlphaBeta
Function AlphaBeta$(what$)
	
	Select what
	
		Case "Alp"	Return "Alpha"
		Case "Bet"	Return "Beta"
		Case "Chi"	Return "Chi"
		Case "Del"	Return "Delta"
		Case "Eps"	Return "Epsilon"
		Case "Eta"	Return "Eta"
		Case "Gam"	Return "Gamma"
		Case "Iot"	Return "Iota"
		Case "Kap"	Return "Kappa"
		Case "Lam"	Return "Lambda"
		Case "My"	Return "My"
		Case "Ny"	Return "Ny"
		Case "Ome"	Return "Omega"
		Case "Omi"	Return "Omicron"
		Case "Phi"	Return "Phi"
		Case "Pi"	Return "Pi"
		Case "Psi"	Return "Psi"
		Case "Rho"	Return "Rho"
		Case "Sig"	Return "Sigma"
		Case "Tau"	Return "Tau"
		Case "The"	Return "Theta"
		Case "Xi"	Return "Xi"
		Case "Yps"	Return "Ypsilon"
		Case "Zet"	Return "Zeta"
			
	End Select
	
End Function

; return long Constellation Name
Function Constellation$(what$)
	
	Select what
			
		Case "And" Return "Andromedae"          ; Andromeda
		Case "Ant" Return "Antliae"	            ; Antlia
        Case "Aps" Return "Apodis"              ; Apus
        Case "Aql" Return "Aquilae"             ; Aquila
        Case "Aqr" Return "Aquarii"             ; Aquarius
        Case "Ara" Return "Arae"                ; Ara
        Case "Ari" Return "Arietis"             ; Aries
        Case "Aur" Return "Aurigae"             ; Auriga
        Case "Boo" Return "Bootis"              ; Bootes
        Case "Cae" Return "Caeli"               ; Caelum
		Case "Cam" Return "Camelopardalis"      ; Camelopardalis
		Case "Cap" Return "Capricorni"          ; Capricornus
		Case "Car" Return "Carinae"             ; Carina
		Case "Cas" Return "Cassiopeiae"         ; Cassiopeia
		Case "Cen" Return "Centauri"            ; Centaurus
		Case "Cep" Return "Cephei"              ; Cepheus
		Case "Cet" Return "Ceti"                ; Cetus
		Case "Cha" Return "Chamaeleontis"       ; Chamaeleon
		Case "Cir" Return "Circini"             ; Circinus
		Case "CMa" Return "Canis Majoris"       ; Canis Major
		Case "CMi" Return "Canis Minoris"       ; Canis Minor
		Case "Cnc" Return "Cancri"              ; Cancer
		Case "Col" Return "Columbae"            ; Columba
		Case "Com" Return "Comae Berenices"     ; Coma Berenices
		Case "CrA" Return "Coronae Austrinae"   ; Corona Australis
		Case "CrB" Return "Coronae Borealis"    ; Corona Borealis
		Case "Crt" Return "Crateris"            ; Crater
		Case "Cru" Return "Crucis"              ; Crux
		Case "Crv" Return "Corvi"               ; Corvus
		Case "CVn" Return "Canum Venaticorum"   ; Canes Venatici
		Case "Cyg" Return "Cygni"               ; Cygnus
		Case "Del" Return "Delphini"            ; Delphinus
		Case "Dor" Return "Doradus"             ; Dorado
		Case "Dra" Return "Draconis"            ; Draco
        Case "Equ" Return "Equulei"             ; Equuleus
		Case "Eri" Return "Eridani"             ; Eridanus
		Case "For" Return "Fornacis"            ; Fornax
		Case "Gem" Return "Geminorum"           ; Gemini
		Case "Gru" Return "Gruis"               ; Grus
		Case "Her" Return "Herculis"            ; Hercules
		Case "Hor" Return "Horologii"           ; Horologium
		Case "Hya" Return "Hydrae"              ; Hydra
		Case "Hyi" Return "Hydri"               ; Hydrus
		Case "Ind" Return "Indi"                ; Indus
		Case "Lac" Return "Lacertae"            ; Lacerta
		Case "Leo" Return "Leonis"              ; Leo
		Case "Lep" Return "Leporis"             ; Lepus
		Case "Lib" Return "Librae"              ; Libra
		Case "LMi" Return "Leonis Minoris"      ; Leo Minor
		Case "Lup" Return "Lupi"                ; Lupus
		Case "Lyn" Return "Lyncis"              ; Lynx
		Case "Lyr" Return "Lyrae"               ; Lyra
		Case "Men" Return "Mensae"              ; Mensa
		Case "Mic" Return "Microscopii"         ; Microscopium
		Case "Mon" Return "Monocerotis"         ; Monoceros
		Case "Mus" Return "Muscae"              ; Musca
		Case "Nor" Return "Normae"              ; Norma
		Case "Oct" Return "Octantis"            ; Octans
		Case "Oph" Return "Ophiuchi"            ; Ophiuchus
		Case "Ori" Return "Orionis"             ; Orion
		Case "Pav" Return "Pavonis"             ; Pavo
		Case "Peg" Return "Pegasi"              ; Pegasus
		Case "Per" Return "Persei"              ; Perseus
		Case "Phe" Return "Phoenicis"           ; Phoenix
		Case "Pic" Return "Pictoris"            ; Pictor
		Case "PsA" Return "Piscis Austrini"     ; Piscis Austrinus
		Case "Psc" Return "Piscium"             ; Pisces
		Case "Pup" Return "Puppis"              ; Puppis
		Case "Pyx" Return "Pyxidis"             ; Pyxis
		Case "Ret" Return "Reticuli"            ; Reticulum
		Case "Scl" Return "Sculptoris"          ; Sculptor
		Case "Sco" Return "Scorpii"             ; Scorpius
		Case "Sct" Return "Scuti"               ; Scutum
		Case "Ser" Return "Serpentis"           ; Serpens
		Case "Sex" Return "Sextantis"           ; Sextans
		Case "Sge" Return "Sagittae"            ; Sagitta
		Case "Sgr" Return "Sagittarii"          ; Sagittarius
		Case "Sol" Return "Solaris"             ; Sol
		Case "Tau" Return "Tauri"               ; Taurus
		Case "Tel" Return "Telescopii"          ; Telescopium
		Case "TrA" Return "Trianguli Australis" ; Triangulum Australe
		Case "Tri" Return "Trianguli"           ; Triangulum
		Case "Tuc" Return "Tucanae"             ; Tucana
		Case "UMa" Return "Ursae Majoris"       ; Ursa Major
		Case "UMi" Return "Ursae Minoris"       ; Ursa Minor
		Case "Vel" Return "Velorum"             ; Vela
		Case "Vir" Return "Virginis"            ; Virgo
		Case "Vol" Return "Volantis"            ; Volans
		Case "Vul" Return "Vulpeculae"          ; Vulpecula
			
	End Select
	
End Function
;~IDEal Editor Parameters:
;~C#Blitz3D